'''
Computes the coefficient matrices and the corresponding headers for solving the 
3S integer linear problem.
'''

import sys
import os
import csv
import math
import json

NUM_SOLVER = 12
TRAIN_DIR = '/public/xcsp_instances/new/dataset/cop/train_test/'
OUT_DIR = '/public/xcsp_instances/new/predictions/cop/3S/'
PORTFOLIOS_FILE = '/public/xcsp_instances/new/scripts/solvers/cop/portfolios_cop'
INFOS_FILE = '/public/xcsp_instances/new/scripts/solvers/cop/infos'
TIMEOUT = 900.0
with open(INFOS_FILE, 'r') as infile:
  infos = json.load(infile)

def get_partial_score(instance, solver, time, lb = 0.25, ub = 0.75):
  item = infos[instance][solver]
  vals = item['vals']
  values = [float(v) for (v, t) in vals.items() if t <= time]
  if not values:
    if item['info'] == 'unsat' and item['otime'] <= time:
      return (1, item['otime'])
    else:
      return (0, time)
  if item['obj'] == 'min':
    val = min(values)
    if val < item['min_val']:
      return (0, time)
  else:
    val = max(values)
    if val > item['max_val']:
      return (0, time)
  score = compute_score(
    item['info'], 
    item['min_val'], 
    item['max_val'], 
    item['obj'], 
    val,
    time,
    item['otime'],
    lb, 
    ub
  )
  t = item['vals'][str(val)]
  if score < 0:
    return (0, t)
  return (score, t)
  
def compute_score(info, min_val, max_val, obj, val, time, otime, lb = 0.25, ub = 0.75):
  if info == "unknown":
    return 0
  if info != "sat" and otime <= time:
    return 1
  if min_val == max_val:
    return ub
  s = ((val - min_val) * (ub - lb)) / (max_val - min_val)
  assert ub - s <= ub or s + lb <= ubs
  if obj == "min":
    return ub - s
  if obj == "max":
    return s + lb
  print "Error!"
  system.exit(1)

def main(args):
  portfolios = csv.reader(open(PORTFOLIOS_FILE), delimiter='|', skipinitialspace = True)
  C = TIMEOUT * 0.1
  for n_solver in range(2, NUM_SOLVER + 1):
    row = portfolios.next()
    solvers = eval(row[1])
    for rep in range(1, 6):
      for fold in range(1, 6):
        name = 'train_rep' + str(rep) + '_fold' + str(fold)
	print "Processing " + name
	instances = csv.reader(open(TRAIN_DIR + name, "rb"), skipinitialspace = True)
	header = open(OUT_DIR + str(n_solver) + '_solvers/header_' + name, 'w')
	coeff  = open(OUT_DIR + str(n_solver) + '_solvers/coeff_'  + name, 'w')     
	insts = []
	M = n_solver
	slots = [set([]) for j in range(0, M)]
	for row in instances:
	  inst = row[0]
	  j = 0
	  ok = False
	  for solver in solvers:
            (score, time) = get_partial_score(inst, solver, C)
	    if score > 0:
	      if time == 0:
		slots[j].add(1.0)
	      else:
		slots[j].add(math.ceil(time))
		ok = True
	    j += 1
	  if ok:
	    insts.append(inst)
	    header.write(inst + ' ')
	P = 0
	for j in range(0, M):
	  s = slots[j]
	  slots[j] = sorted(s)
	  P += len(s)
	print "Computed instances solvable in " + str(C) + " seconds and time slots for each solver"
	
	N = len(insts)
	A = [[0] * (N + P) for i in range(N + 1)]
	for i in range(0, N):
	  A[i][i] = 1
	  k = N
	  for j in range(0, M):
	    for t in slots[j]:
              if get_partial_score(insts[i], solvers[j], t)[0] > 0:
		A[i][k] = 1
	      A[N][k] = t
	      k += 1
	      if i == 0:
		header.write(solvers[j] + '_' + str(t) + ' ')
	print "Computed coefficient matrix..."
	for i in range(0, N + 1):
	  for j in range(0, N + P):
	      coeff.write(str(A[i][j]) + ' ')
	  coeff.write('\n')
	print "Completed " + name
    header.close()
    coeff.close()

if __name__ == '__main__':
    main(sys.argv[1:])
