'''
Extracts infos for off-the-shelf classifiers.
'''
import sys
import os
import json

BACKUP_SOLVER = 'g12cpx'
FEAT_SOLVER = 'gecode'
TIMEOUT = 900.0
infos_file = '/public/xcsp_instances/new/scripts/solvers/cop/infos'
results_file = '/public/xcsp_instances/new/scripts/classifiers/cop/mznc/results'
in_path = '/public/xcsp_instances/new/predictions/cop/off_the_shelf/pred_no_sel/'

def get_score(info, min_val, max_val, obj, val, time, otime, lb = 0.25, ub = 0.75):
  if info == "unknown":
    return 0
  if info != "sat" and otime <= time:
    return 1
  if min_val == max_val:
    return ub
  s = ((val - min_val) * (ub - lb)) / (max_val - min_val)
  #assert 0 <= round(s, 5) <= ub - lb
  assert ub - s <= 1 or s + lb <= 1
  if obj == "min":
    return ub - s
  if obj == "max":
    return s + lb
  print "Error!"
  system.exit(1) 

def get_partial_score(instance, solver, time, infos, lb = 0.25, ub = 0.75):
  item = infos[instance][solver]
  vals = item['vals']
  values = [float(v) for (v, t) in vals.items() if float(t) <= time]
  if not values:
    if item['info'] == 'unsat' and item['otime'] <= time:
      return 1
    else:
      return 0
  if item['obj'] == 'min':
    val = min(values)
    if val < item['min_val']:
      return 0
  else:
    val = max(values)
    if val > item['max_val']:
      return 0
  score = get_score(
    item['info'], 
    item['min_val'], 
    item['max_val'], 
    item['obj'], 
    val,
    time,
    item['otime'],
    lb, 
    ub
  )
  if score < 0:
    return 0
  return score

def compute_predictions(pred_path, classifier, no_solvers, infos, results):
  dir_list = sorted(os.listdir(pred_path))
  for file_name in dir_list:
    f = open(pred_path + "/" + file_name)
    rep = file_name[file_name.index('_rep') + 4]
    fold = file_name[file_name.index('_fold') + 5]
    print classifier, no_solvers, rep, fold
    # Skip the first line.
    f.readline() 
    line = f.readline()
    while line:
      row = line.split()
      if len(row) > 0:
        # Get the predicted solver and the instance name.
        solver = row[2].split(':')[1]
        instance = row[-1][1:-1]
        if solver == 'minisati' or solver == 'minisat':
          solver = 'minisatid'
        elif solver == 'g12lazyf' or solver == 'g12lazy':
          solver = 'g12lazyfd'
        elif solver == 'nosolver' or solver == 'nosolve':
          solver = BACKUP_SOLVER
        item = infos[instance][solver]
        time = item['feat_time']
	if solver == FEAT_SOLVER:
	  time -= item['fzn_time']
	if item['otime'] + time <= TIMEOUT:
	  score = get_partial_score(instance, solver, TIMEOUT, infos)
	  score01 = get_partial_score(instance, solver, TIMEOUT, infos, 0, 1)
	else:
	  score = get_partial_score(instance, solver, TIMEOUT - time, infos)
	  score01 = get_partial_score(instance, solver, TIMEOUT - time, infos, 0, 1)
	otime = time + item['otime']
	if score < 1:
	  otime = TIMEOUT
	if no_solvers not in results.keys():
	  results[no_solvers] = {}
	if instance not in results[no_solvers].keys():
	  results[no_solvers][instance] = {}
	if rep not in results[no_solvers][instance].keys():
	  results[no_solvers][instance][rep] = {}
	if score == 1:
	  opt = 1
	else:
	  opt = 0
	assert 0 <= score <= 1
	assert 0 <= otime <= TIMEOUT
	assert score == 1 or otime == TIMEOUT
	results[no_solvers][instance][rep][classifier] = (score, opt, otime, None, score01)
	# For following use.
	for c in ['sunny', '3s', 'satzilla']:
	  results[no_solvers][instance][rep][c] = None
      line = f.readline()
        
def main(args):
  results = {}
  with open(infos_file, 'r') as infile:
    infos = json.load(infile)
  for c in ['RF', 'SMO']:
    for n in range(2, 13):
      path = in_path + str(n) + '_solvers/' + c +'/no-oversamp/RemNorm/no_sel'  
      cp = compute_predictions(path, c, str(n), infos, results)
  with open(results_file, 'w') as outfile:
    json.dump(results, outfile)

if __name__ == '__main__':
    main(sys.argv[1:])
