'''
Computes statistics for SATzilla
'''

import csv
import sys
import os
import json

BACKUP_SOLVER = 'g12cpx'
FEAT_SOLVER = 'gecode'
TIMEOUT = 900.0
NUM_SOLVER = 12
FEAT_PATH = '/public/xcsp_instances/new/scripts/classifiers/cop/SATzilla/features/'
PRED_PATH = '/public/xcsp_instances/new/predictions/cop/SATzilla/'
TEST_PATH = '/public/xcsp_instances/new/predictions/cop/SATzilla/'
INFOS = '/public/xcsp_instances/new/scripts/solvers/cop/infos'

def get_score(info, min_val, max_val, obj, val, time, otime, lb = 0.25, ub = 0.75):
  if info == "unknown":
    return 0
  if info != "sat" and otime <= time:
    return 1
  if min_val == max_val:
    return ub
  s = ((val - min_val) * (ub - lb)) / (max_val - min_val)
  #assert 0 <= round(s, 5) <= ub - lb
  assert ub - s <= 1 or s + lb <= 1
  if obj == "min":
    return ub - s
  if obj == "max":
    return s + lb
  print "Error!"
  system.exit(1) 

def get_partial_score(instance, solver, time, infos, lb = 0.25, ub = 0.75):
  item = infos[instance][solver]
  vals = item['vals']
  values = [float(v) for (v, t) in vals.items() if float(t) <= time]
  if not values:
    if item['info'] == 'unsat' and item['otime'] <= time:
      return 1
    else:
      return 0
  if item['obj'] == 'min':
    val = min(values)
    if val < item['min_val']:
      return 0
  else:
    val = max(values)
    if val > item['max_val']:
      return 0
  score = get_score(
    item['info'], 
    item['min_val'], 
    item['max_val'], 
    item['obj'], 
    val,
    time,
    item['otime'],
    lb, 
    ub
  )
  if score < 0:
    return 0
  return score

def main(args):
  with open(INFOS, 'r') as infile:
    infos = json.load(infile)
  res_path = '/public/xcsp_instances/new/scripts/classifiers/cop/mznc/results'
  with open(res_path, 'r') as infile:
    results = json.load(infile)
    infile.close()
  for n in range(2, NUM_SOLVER + 1):
    for rep in range(1, 6):
      m = 0
      for fold in range(1, 6):
        suffix = 'rep' + str(rep) + '_fold' + str(fold)
        feat = FEAT_PATH + 'test_' + suffix + '.features'
        test = TEST_PATH + str(n) + '_solvers/test_' + str(n) + 'solvers_' + suffix
        feat_reader = csv.reader(open(feat, 'rb'),  delimiter = ' ')
        test_reader = csv.reader(open(test, 'rb'),  delimiter = ' ')
        for row in feat_reader:
	  m += 1
          instance = row[0]
          solver = test_reader.next()[0]
          item = infos[instance][solver]
          time = item['feat_time']
          if solver == FEAT_SOLVER:
            time -= item['fzn_time']
	  if item['otime'] + time <= TIMEOUT:
	    score = get_partial_score(instance, solver, TIMEOUT, infos)
	    score01 = get_partial_score(instance, solver, TIMEOUT, infos, 0, 1)
	  else:
	    score = get_partial_score(instance, solver, TIMEOUT - time, infos)
	    score01 = get_partial_score(instance, solver, TIMEOUT, infos, 0, 1)
	  if score == 1:
	    otime = time + item['otime']
	    opt = 1
	  else:
	    otime = TIMEOUT
	    opt = 0
	  assert 0 <= score <= 1
	  assert 0 <= otime <= TIMEOUT
	  assert score == 1 or otime == TIMEOUT
          results[str(n)][instance][str(rep)]['satzilla'] = (score, opt, otime, None, score01)
      assert m == 2670
  with open(res_path, 'w') as outfile:
    json.dump(results, outfile)       

if __name__ == '__main__':
    main(sys.argv[1:])
