'''
Compute statistics for off-the-shelf classifiers (called by compute_statistics_ots).
'''
import sys
import os
import json

BACKUP_SOLVER = 'g12cpx'
FEAT_SOLVER = 'gecode'
TIMEOUT = 900.0
infos_file = '/public/xcsp_instances/new/scripts/solvers/cop/infos'

def get_score(info, min_val, max_val, obj, val, time, otime, lb = 0.25, ub = 0.75):
  if info == "unknown":
    return 0
  if info != "sat" and otime <= time:
    return 1
  if min_val == max_val:
    return ub
  s = ((val - min_val) * (ub - lb)) / (max_val - min_val)
  #assert 0 <= round(s, 5) <= ub - lb
  assert ub - s <= 1 or s + lb <= 1
  if obj == "min":
    return ub - s
  if obj == "max":
    return s + lb
  print "Error!"
  system.exit(1) 

def get_partial_score(instance, solver, time, infos, lb = 0.25, ub = 0.75):
  item = infos[instance][solver]
  vals = item['vals']
  values = [float(v) for (v, t) in vals.items() if float(t) <= time]
  if not values:
    if item['info'] == 'unsat' and item['otime'] <= time:
      return 1
    else:
      return 0
  if item['obj'] == 'min':
    val = min(values)
    if val < item['min_val']:
      return 0
  else:
    val = max(values)
    if val > item['max_val']:
      return 0
  score = get_score(
    item['info'], 
    item['min_val'], 
    item['max_val'], 
    item['obj'], 
    val,
    time,
    item['otime'],
    lb, 
    ub
  )
  if score < 0:
    return 0
  return score


def compute_predictions(pred_path, infos):
  ranges = [(0.25, 0.75), (0.2, 0.8), (0.1, 0.9),  (0, 1)]
  asp = {}
  for (a, b) in ranges:
    asp[str(a) + '_' + str(b)] = 0.0
  pof = 0.0
  aot = 0.0
  counter = 0
  dir_list = sorted(os.listdir(pred_path))
    
  for file_name in dir_list:
    f = open(pred_path + "/" + file_name)
    # Skip the first line.
    f.readline() 
    line = f.readline()
    while line:
      row = line.split()
      if len(row) > 0:
        counter += 1
        # Get the predicted solver and the instance name.
        solver = row[2].split(':')[1]
        instance = row[-1][1:-1]
        if solver == 'minisati' or solver == 'minisat':
          solver = 'minisatid'
        elif solver == 'g12lazyf' or solver == 'g12lazy':
          solver = 'g12lazyfd'
        elif solver == 'nosolver' or solver == 'nosolve':
          solver = BACKUP_SOLVER
        item = infos[instance][solver]
        #score = item['score']
        #area = item['area']
        time = item['feat_time']
	if solver == FEAT_SOLVER:
	  time -= item['fzn_time']
	for (a, b) in ranges:
	  if item['otime'] + time <= TIMEOUT:
	    score = get_partial_score(instance, solver, TIMEOUT, infos, a, b)
	  else:
	    score = get_partial_score(instance, solver, TIMEOUT - time, infos, a, b)
	  asp[str(a) + '_' + str(b)] += score
	  if a == 0.25 and b == 0.75:
	    sc = score
	if sc == 1:
	  aot += time + item['otime']
	  pof += 1
	else:
	  aot += TIMEOUT
      line = f.readline()
  for (a, b) in ranges:
    asp[str(a) + '_' + str(b)] = str(100 * float(asp[str(a) + '_' + str(b)]) / counter)
  pof = str(100 * float(pof) / counter)
  aot = str(aot / counter)
  s = ''
  for (a, b) in ranges:
    s += asp[str(a) + '_' + str(b)] + '|'
  return s + pof + '|' + aot 
        
def main(args):
  with open(infos_file, 'r') as infile:
    infos = json.load(infile)
  cp = compute_predictions(args[0], infos)
  print cp

if __name__ == '__main__':
    main(sys.argv[1:])
