'''
Compute the elections with different criteria (viz. Approval, Borda, Plurality)
'''
import sys
import csv
import json
import math

times_file='/public/xcsp_instances/new/scripts/solvers/cop/infotimevals_cop'
feat_file = '/public/xcsp_instances/new/features/cop/features.csv'
info_file = '/public/xcsp_instances/new/scripts/solvers/cop/infos'
solvers_rank = []
num_solvers = 12
results = {}
feat_vector = {}
infos = {}
voters = 0
abstained = 0
FEAT_INST = 0
FEAT_GOAL = 109 - 1
FEAT_FZN  = 153 - 1
FEAT_TIME = 155 - 1
TIMEOUT = 900.0 #1800.0

def init_results():
  global results, voters, abstained
  voters = 0
  abstained = 0
  results = {
    'bprolog': 0, 'fzn2smt': 0, 'g12cpx': 0, 'g12fd': 0, 'g12lazyfd': 0, 
    'g12mip': 0, 'gecode': 0, 'izplus': 0, 'jacop': 0, 'minisatid': 0, 
    'mistral': 0, 'ortools': 0
  }
  
def compute_score(info, min_val, max_val, obj, val, lb = 0.25, ub = 0.75):
  if info == "unknown":
    return 0
  if info != "sat":
    return 1
  if val == min_val == max_val:
    return ub
  s = ((val - min_val) * (ub - lb)) / (max_val - min_val)
  assert 0 <= round(s, 5) <= (ub - lb)
  if obj == "min":
    return ub - s
  if obj == "max":
    return s + lb
  print "Error!"
  system.exit(1)
    
def scale(val, goal, min_val, max_val, lb, ub):
  if val == min_val == max_val:
    return lb
  s = (ub - lb) * float(val - min_val) / (max_val - min_val)
  assert 0 <= round(s, 5) <= (ub - lb)
  if goal == 'min':
    return lb + s
  else:
    return ub - s    
    
def compute_area(solver_info, goal, min_val, max_val, lb = 0.25, ub = 0.75):
  info = solver_info['info']
  time = solver_info['otime']
  if info == 'unsat':
    return time
  if info == 'unknown':
    return TIMEOUT
  scaled_vals = sorted([
    (float(t), scale(float(v), goal, min_val, max_val, lb, ub))
    for (v, t) in solver_info['vals'].items() if float(t) < TIMEOUT
  ])
  n = len(scaled_vals) - 1
  a = scaled_vals[0][0]
  for i in range(0, n):
    a += scaled_vals[i][1] * (scaled_vals[i + 1][0] - scaled_vals[i][0])
  a += scaled_vals[n][1] * (time - scaled_vals[n][0])
  assert time >= scaled_vals[n][0]
  assert 0 <= round(a, 5) <= TIMEOUT
  return a
    
def fix_objective(inst):
  if 'pattern_set_mining_k1' in inst:
    return 'max'
  elif 'radiation' in inst:
    return 'min'
  else:
    print 'Unknown instance:',inst, int(feat_vector[inst][FEAT_GOAL])
    sys.exit(0)

def approval_rule(inst):
  global voters, abstained, feat_vector
  ranges = [(0.1, 0.9), (0.2, 0.8), (0, 1)]
  solvers = infos[inst]
  approval_list = filter(lambda item: item[1]['info'] != 'unknown', solvers.items())
  if int(feat_vector[inst][FEAT_GOAL]) == 2:
    obj = 'min'
  elif int(feat_vector[inst][FEAT_GOAL]) == 3:
    obj = 'max'
  else:
    obj = fix_objective(inst)
  for solver in solvers.keys():
    solvers[solver]['obj'] = obj
    solvers[solver]['fzn_time']  = float(feat_vector[inst][FEAT_FZN])
    solvers[solver]['feat_time'] = float(feat_vector[inst][FEAT_TIME])
    if not approval_list:
      solvers[solver]['score'] = 0
      solvers[solver]['area'] = TIMEOUT
      for (a, b) in ranges:
	solvers[solver]['score_' + str(a) + '_' + str(b)] = 0
	solvers[solver]['area_' + str(a) + '_' + str(b)] = TIMEOUT
      solvers[solver]['otime'] = TIMEOUT
      solvers[solver]['min_val'] = float('nan')
      solvers[solver]['max_val'] = float('nan')
      solvers[solver]['min_vals'] = float('nan')
      solvers[solver]['max_vals'] = float('nan')
  if approval_list:
    voters += 1
    min_val = min([solvers[solver]['val'] for (solver, _) in approval_list])
    max_val = max([solvers[solver]['val'] for (solver, _) in approval_list])
    vals = [
      float(v) for (solver, _) in approval_list 
      for (v, t) in solvers[solver]['vals'].items() 
      if float(t) < TIMEOUT
    ]
    if vals:
      min_vals = min(vals)
      max_vals = max(vals)
    else:
      min_vals = float('nan')
      max_vals = float('nan')
    for solv in solvers.keys():
      solvers[solv]['min_val'] = min_val
      solvers[solv]['max_val'] = max_val
      solvers[solv]['min_vals'] = min_vals
      solvers[solv]['max_vals'] = max_vals
      if solv in [s for (s, i) in approval_list]:
        item = solvers[solv]
        score = compute_score(item['info'], min_val, max_val, obj, item['val'])
        results[solv] += score
        item['score']  = score
        item['area'] = compute_area(item, obj, min_vals, max_vals)
        for (a, b) in ranges:
	  solvers[solv]['score_' + str(a) + '_'  + str(b)] = \
	    compute_score(item['info'], min_val, max_val, obj, item['val'], a, b)
	  solvers[solv]['area_' + str(a) + '_'  + str(b)] = \
	    compute_area(item, obj, min_vals, max_vals, a, b)
      else:
        solvers[solv]['score'] = 0
        solvers[solv]['area'] = TIMEOUT
        for (a, b) in ranges:
	  solvers[solv]['score_' + str(a) + '_' + str(b)] = 0
	  solvers[solv]['area_' + str(a) + '_' + str(b)] = TIMEOUT
        solvers[solv]['otime']  = TIMEOUT
  else:
    abstained += 1
      
def borda_rule(inst):
  global voters, abstained
  solvers = infos[inst]
  approval_list = filter(lambda item: item[1]['info'] != 'unknown', solvers.items())
  if not approval_list:  
    abstained += 1
    return
  voters += 1
  times = []
  for (solv, _) in approval_list:
    score = solvers[solv]['score']
    time  = solvers[solv]['otime']
    times.append((1 - score, time, solvers_rank.index(solv), solv))
  rank = sorted(times)
  global num_solvers
  n = num_solvers
  for item in rank:
    results[item[3]] += n
    n -= 1
  
def plurality_rule(inst):
  global voters, abstained
  solvers = infos[inst]
  approval_list = filter(lambda item: item[1]['info'] != 'unknown', solvers.items())
  if not approval_list:  
    abstained += 1
    return
  voters += 1
  times = []
  for (solv, _) in approval_list:
    score = solvers[solv]['score']
    time  = solvers[solv]['otime']
    times.append((1 - score, time, solvers_rank.index(solv), solv))
  best = min(times)[3]
  global num_solvers
  results[best] += 1

def better_ans(a_i, a_j):
  return a_i == 'opt' and a_j != 'opt' \
      or a_i == 'sat' and a_j in ['unsat', 'unk'] \
      or a_i == 'unsat' and a_j in ['sat', 'unk']

def better_val(v_i, v_j, goal):
  return goal == 'min' and v_i < v_j or goal == 'max' and v_i > v_j  
  
def mzc_rule(inst, item):
  solvers = item.keys()
  n = len(solvers)
  score = dict((s, 0) for s in solvers)
  for i in range(0, n - 1):
    s_i = solvers[i]
    obj = item[s_i]['obj']
    time_i = math.floor(item[s_i]['otime'])
    ans_i = item[s_i]['info']
    val_i = item[s_i]['val']
    for j in range(i + 1, n):
      s_j = solvers[j]
      time_j = math.floor(item[s_j]['otime'])
      ans_j = item[s_j]['info']
      val_j = item[s_j]['val']
      if better_ans(ans_i, ans_j):
        results[s_i] += 1
        score[s_i] += 1
      elif better_ans(ans_j, ans_i):
	results[s_j] += 1
	score[s_j] += 1
      else:
        if ans_i != 'unknown':
          if time_i == 0:
            results[s_i] += 0.5
            score[s_i] += 0.5
          else:
            results[s_i] += time_j / (time_i + time_j)
            score[s_i] += time_j / (time_i + time_j)
        if ans_j != 'unknown':
	  if time_j == 0:
            results[s_j] += 0.5
            score[s_j] += 0.5
          else:
            results[s_j] += time_i / (time_i + time_j)
            score[s_j] += time_i / (time_i + time_j)
  for s in solvers:
    assert 0 <= score[s] <= n - 1
    item[s]['mzc_score'] = score[s]
  
def main(args):
  feat_reader  = csv.reader(open(feat_file,  'rb'), delimiter = '|')
  feat_vect_file = '/public/xcsp_instances/new/features/cop/feat_vector'
  for row in feat_reader:
    feat_vector[row[0]] = row[1 : len(row)]
  with open(feat_vect_file, 'w') as outfile:
    json.dump(feat_vector, outfile)
  
  times_reader = csv.reader(open(times_file, 'rb'), delimiter = '|')
  init_results()
  global infos, TIMEOUT
  for row in times_reader:
    inst = row[0]
    infos[inst] = {}
    solvers = eval(row[1])
    for (solver, (info, time, val, vals)) in solvers.items():
      infos[inst][solver] = {}
      infos[inst][solver]['info'] = info
      time = float(time)
      vals = dict([
	(float(v), float(t)) for (v, t) in vals.items() 
	if float(t) < TIMEOUT
      ])
      if info in ['sat', 'unknown'] or time >= TIMEOUT:
	infos[inst][solver]['otime'] = TIMEOUT
      else:
	if vals:
	  if time < max(t for t in vals.values()):
	    time = max(t for t in vals.values())
	elif info == 'opt':
	  time = TIMEOUT
	infos[inst][solver]['otime'] = time
      if vals:
	if int(feat_vector[inst][FEAT_GOAL]) == 2:
	  obj = 'min'
	elif int(feat_vector[inst][FEAT_GOAL]) == 3:
	  obj = 'max'
	else:
	  obj = fix_objective(inst)
	if obj == 'min':
	  infos[inst][solver]['val'] = min(vals)
	else:
	  infos[inst][solver]['val'] = min(vals)
      else:
        infos[inst][solver]['val'] = float('nan')
      infos[inst][solver]['vals'] = vals
      if infos[inst][solver]['otime'] == TIMEOUT:
	if vals:
	  infos[inst][solver]['info'] = 'sat'
	else:
	  infos[inst][solver]['info'] = 'unknown'
      assert infos[inst][solver]['info'] in ['unsat', 'unknown'] or infos[inst][solver]['vals']
    approval_rule(inst)
  approval_res = \
    sorted(results.items(), key = lambda item: item[1], reverse = True)
  print "Approval:", approval_res 
  print "Voters:", voters
  print "Abstained:", abstained
  print "Total:", voters + abstained
  print "----------\n"
  with open(info_file, 'w') as outfile:
    json.dump(infos, outfile)
  global solvers_rank
  solvers_rank = [item[0] for item in approval_res]
  
  init_results()
  for inst in infos.keys():
    borda_rule(inst)
  print "Borda:", \
    sorted(results.items(), key = lambda item: item[1], reverse = True)
  print "Voters:", voters
  print "Abstained:", abstained
  print "Total:", voters + abstained
  print "----------\n"
  
  init_results()
  for inst in infos.keys():
    plurality_rule(inst)
  print "Plurality:", \
    sorted(results.items(), key = lambda item: item[1], reverse = True)
  print "Voters:", voters
  print "Abstained:", abstained
  print "Total:", voters + abstained
  print "----------\n"
  
  init_results()
  for inst in infos.keys():
    mzc_rule(inst, infos[inst])
  print "MZC score:", \
    sorted(results.items(), key = lambda item: item[1], reverse = True)
  print "----------\n"
  
if __name__ == '__main__':
    main(sys.argv[1:])
