'''
Builds portfolios of different size m = 2, ..., 12.
'''

import csv
import sys
import json
from combinations import *

scores = {
  'bprolog': [], 'fzn2smt': [], 'g12cpx': [], 'g12fd': [], 
  'g12lazyfd': [], 'g12mip': [], 'gecode': [], 'izplus': [], 
  'jacop': [], 'minisatid': [], 'mistral': [], 'ortools': []
}
times = {
  'bprolog': 0, 'fzn2smt': 0, 'g12cpx': 0, 'g12fd': 0, 
  'g12lazyfd': 0, 'g12mip': 0, 'gecode': 0, 'izplus': 0, 
  'jacop': 0, 'minisatid': 0, 'mistral': 0, 'ortools': 0
}
solvers = scores.keys()
num_solvers = len(solvers)
infos_file = '/public/xcsp_instances/new/scripts/solvers/cop/infos'
portfolios_file = '/public/xcsp_instances/new/scripts/solvers/cop/portfolios_cop'
portfolios_writer = csv.writer(open(portfolios_file, 'wb'),  delimiter = '|')
best_portfolios = {}
infos = {}

def compute_score(info, min_val, max_val, obj, val, lb = 0.25, ub = 0.75):
  if info == "unknown":
    return 0
  if info != "sat":
    return 1
  if val == min_val == max_val:
    return ub
  s = ((val - min_val) * (ub - lb)) / (max_val - min_val)
  assert 0 <= round(s, 5) <= (ub - lb)
  if obj == "min":
    return ub - s
  if obj == "max":
    return s + lb
  print "Error!"
  system.exit(1)

def compute_portfolio(k):
  max_score = 0
  min_time = 1000000000
  for j in range(0, binom(num_solvers, k)):
    portfolio = get_subset(j, k, solvers)
    port_scores = dict([(s, l) for s, l in scores.items() if s in portfolio])
    score = 0
    num_inst = len(port_scores[portfolio[0]])
    for i in range(0, num_inst):
      score += max([inst[i] for inst in port_scores.values()])
    solving_time = sum([times[solver] for solver in portfolio])
    if score >  max_score or \
      (score == max_score and solving_time < min_time):
        min_time  = solving_time
        max_score = score
        best_portfolio = portfolio
  best_portfolio = sorted(best_portfolio, key = lambda s: scores[s], reverse = True)
  print '---> Best portfolio',best_portfolio,'( score = ',max_score,')'
  portfolios_writer.writerow([k, best_portfolio])

def main(args):
  global infos
  print 'Computing solved instances...'
  with open(infos_file, 'r') as infile:
    infos = json.load(infile)
  for (inst, item) in infos.items():
    for (solver, info) in item.items():
      scores[solver].append(infos[inst][solver]['score']) 
      times[solver]  += infos[inst][solver]['otime']
  print 'Computing portfolios...'
  for i in range(2, num_solvers + 1):
    print '--- Computing portfolios of',i,'solvers:'
    compute_portfolio(i)

if __name__ == '__main__':
    main(sys.argv[1:])
