'''
Compute statistics for single solvers and VBS.
'''
import sys
import os
import csv
import json

solvers = [
  'bprolog', 'fzn2smt', 'g12cpx', 'g12fd', 'g12lazyfd', 'g12mip', 'gecode', 
  'izplus', 'jacop', 'minisatid', 'mistral', 'ortools'
]
TIMEOUT = 900.0
infos_file = '/public/xcsp_instances/new/scripts/solvers/cop/infos'
portfolios_file = '/public/xcsp_instances/new/scripts/solvers/cop/portfolios_cop'

def compute_stats(solver, infos):
  asp = {}
  ava = {}
  asp['0.25_0.75'] = 0.0
  ava['0.25_0.75'] = 0.0
  pof = 0.0
  aot = 0.0
  ranges = [(0.2, 0.8), (0.1, 0.9),  (0, 1)]  
  for (a, b) in ranges:
    asp[str(a) + '_' + str(b)] = 0.0
    ava[str(a) + '_' + str(b)] = 0.0
  counter = 0
  for (inst, item) in sorted(infos.items()):
    counter += 1
    score = item[solver]['score']
    time  = item[solver]['otime']
    area  = item[solver]['area']
    for (a, b) in ranges:
      asp[str(a) + '_' + str(b)] += item[solver]['score_' + str(a) + '_' + str(b)]
      ava[str(a) + '_' + str(b)] += item[solver]['area_' + str(a) + '_' + str(b)]
    asp['0.25_0.75'] += score
    ava['0.25_0.75'] += area
    if score > 0:
      if score == 1:
	aot += time
	pof += 1
      else:
	aot += TIMEOUT
    else:
      aot += TIMEOUT
  pof = str(100 * float(pof) / counter)
  aot = str(aot / counter)
  for (a, b) in ranges:
    asp[str(a) + '_' + str(b)] = str(100 * float(asp[str(a) + '_' + str(b)]) / counter)
    ava[str(a) + '_' + str(b)] = str(float(ava[str(a) + '_' + str(b)]) / counter)
  s = solver + '|' + str(100 * float(asp['0.25_0.75']) / counter) + '|'
  for (a, b) in ranges:
    s += asp[str(a) + '_' + str(b)] + '|'
  s += str(float(ava['0.25_0.75']) / counter) + '|'
  for (a, b) in ranges:
    s += ava[str(a) + '_' + str(b)] + '|'
  return s + pof + '|' + aot

def compute_stats_vbs(portfolio, infos):
  asp = {}
  ava = {}
  asp['0.25_0.75'] = 0.0
  ava['0.25_0.75'] = 0.0
  pof = 0.0
  aot = 0.0
  ranges = [(0.2, 0.8), (0.1, 0.9),  (0, 1)]  
  for (a, b) in ranges:
    asp[str(a) + '_' + str(b)] = 0.0
    ava[str(a) + '_' + str(b)] = 0.0
  counter = 0
  for (inst, item) in sorted(infos.items()):
    counter += 1
    port_infos = [
      (1 - float(item[s]['score']), 
           float(item[s]['otime']), 
           s) 
      for s in portfolio
    ]
    rank = sorted(port_infos) 
    (score, time, s) = rank[0]
    score = 1 - score
    area  = item[s]['area']
    for (a, b) in ranges:
      asp[str(a) + '_' + str(b)] += item[s]['score_' + str(a) + '_' + str(b)]
      ava[str(a) + '_' + str(b)] += item[s]['area_' + str(a) + '_' + str(b)]
    asp['0.25_0.75'] += score
    ava['0.25_0.75'] += area
    if score > 0:
      if score == 1:
	aot += time
	pof += 1
      else:
	aot += TIMEOUT
    else:
      aot += TIMEOUT
  pof = str(100 * float(pof) / counter)
  aot = str(aot / counter)
  for (a, b) in ranges:
    asp[str(a) + '_' + str(b)] = str(100 * float(asp[str(a) + '_' + str(b)]) / counter)
    ava[str(a) + '_' + str(b)] = str(float(ava[str(a) + '_' + str(b)]) / counter)
  st = 'vbs_' + str(len(portfolio)) + '|' + str(100 * float(asp['0.25_0.75']) / counter) + '|'
  for (a, b) in ranges:
    st += asp[str(a) + '_' + str(b)] + '|'
  st += str(float(ava['0.25_0.75']) / counter) + '|'
  for (a, b) in ranges:
    st += ava[str(a) + '_' + str(b)] + '|'
  return st + pof + '|' + aot
     
def main(args):
  with open(infos_file, 'r') as infile:
    infos = json.load(infile)
  print 'solver|asp|pof|psi|aot|ast'
  for solver in solvers:
    cp = compute_stats(solver, infos)
    print cp
  portfolios_reader = csv.reader(open(portfolios_file,  'rb'),  delimiter = '|')
  for row in portfolios_reader:
    cp = compute_stats_vbs(eval(row[1]), infos)
    print cp

if __name__ == '__main__':
    main(sys.argv[1:])

